package digital.laboratory.platform.reagent.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.annotation.JsonFormat;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.config.QRCodeUtils;
import digital.laboratory.platform.reagent.entity.ReagentConsumableInventory;
import digital.laboratory.platform.reagent.entity.ReferenceMaterial;
import digital.laboratory.platform.reagent.service.ReagentConsumableInventoryService;
import digital.laboratory.platform.reagent.service.ReferenceMaterialService;
import digital.laboratory.platform.reagent.vo.*;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.security.Principal;
import java.sql.Date;
import java.util.ArrayList;
import java.util.List;

/**
 * 试剂耗材库存
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 试剂耗材库存 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/reagent_consumable_inventory")
@Api(value = "reagent_consumable_inventory", tags = "试剂耗材库存管理")
public class ReagentConsumableInventoryController {

    private final ReagentConsumableInventoryService reagentConsumableInventoryService;

    private final ReferenceMaterialService referenceMaterialService;

    private final OssFile ossFile;

    /**
     * 通过id查询试剂耗材库存
     *
     * @param reagentConsumableInventoryId id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{reagentConsumableInventoryId}")
//    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')")
    public R<ReagentConsumableInventory> getById(@PathVariable("reagentConsumableInventoryId") String reagentConsumableInventoryId, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.getById(reagentConsumableInventoryId);
        return R.ok(reagentConsumableInventory);
        //return R.ok(reagentConsumableInventoryService.getById(reagentConsumableInventoryId));
    }


    /**
     * @param
     * @return
     */
    @ApiOperation(value = "标准物质列表", notes = "标准物质列表")
    @GetMapping("/RList")
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')" )
    public R<Page<ReagentConsumableInventoryFullVO>> getReagentConsumableInventoryPage(Integer current, Integer size, String reagentConsumableName, Integer referenceMaterialStatus, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();
        QueryWrapper<ReferenceMaterial> referenceMaterialQueryWrapper = new QueryWrapper<>();

        if (reagentConsumableName != null) {

            reagentConsumableInventoryQueryWrapper.like("reagent_consumable_name", reagentConsumableName);

        }


        Page<ReagentConsumableInventoryFullVO> allList = reagentConsumableInventoryService.getAllList(current, size, reagentConsumableInventoryQueryWrapper, referenceMaterialStatus);
        return R.ok(allList);
//		return R.ok(reagentConsumableInventoryService.page(page, Wrappers.query(reagentConsumableInventory)));
    }

    @ApiOperation(value = "标准物质管理列表", notes = "标准物质管理列表")
    @GetMapping("/standardList")
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')" )
    public R<IPage<ReagentConsumableInventoryVO>> getReagentConsumableInventoryList(Page<ReagentConsumableInventory> page, String reagentConsumableName, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        IPage<ReagentConsumableInventoryVO> reagentConsumableInventoryRMVOList = reagentConsumableInventoryService.getReagentConsumableInventoryRMVOList(page, Wrappers.<ReagentConsumableInventory>query()
                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .eq("category", "标准物质").or()
                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .eq("category", "标准储备溶液"));
        return R.ok(reagentConsumableInventoryRMVOList);
//		return R.ok(reagentConsumableInventoryService.page(page, Wrappers.query(reagentConsumableInventory)));
    }

    @ApiOperation(value = "试剂耗材管理列表", notes = "试剂耗材管理列表")
    @GetMapping("/List")
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')" )
    public R<IPage<ReagentConsumableInventoryVO>> getList(Page page, String reagentConsumableName, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();
        IPage<ReagentConsumableInventoryVO> reagentConsumableInventoryRMVOList = reagentConsumableInventoryService.getReagentConsumableInventoryRMVOList(page, Wrappers.<ReagentConsumableInventory>query()
                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .eq("category", "试剂").or()
                .like(StrUtil.isNotBlank(reagentConsumableName), "reagent_consumable_name", reagentConsumableName)
                .eq("category", "耗材"));
        return R.ok(reagentConsumableInventoryRMVOList);
//		return R.ok(reagentConsumableInventoryService.page(page, Wrappers.query(reagentConsumableInventory)));
    }

    /**
     * 分页查询
     *
     * @return
     */
    @ApiOperation(value = "试剂耗材列表", notes = "试剂耗材列表")
    @GetMapping("/MList")
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')" )
    public R<IPage<ReagentConsumableInventoryFullVO>> getReferenceMaterialVOList(Page page, String reagentConsumableName, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();

        IPage<ReagentConsumableInventoryFullVO> allRM = reagentConsumableInventoryService.getAllRM(page, reagentConsumableInventoryQueryWrapper
                .eq("category", "试剂")
                .like(!StrUtil.isEmpty(reagentConsumableName), "reagent_consumable_name", reagentConsumableName).or()
                .eq("category", "耗材")
                .like(!StrUtil.isEmpty(reagentConsumableName), "reagent_consumable_name", reagentConsumableName));


        return R.ok(allRM);
    }


    /**
     * 分页查询
     *
     * @return
     */
    @ApiOperation(value = "试剂耗材/标准物质集合列表", notes = "试剂耗材/标准物质集合列表")
    @GetMapping("/full")
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')" )
    public R<List<ReagentConsumableInventoryFullVO>> getReagentConsumableInventoryFull(String category, String name, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        QueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryQueryWrapper = new QueryWrapper<>();

        if (category != null) {
            reagentConsumableInventoryQueryWrapper.eq("category", category);
        }
        if (name != null) {
            reagentConsumableInventoryQueryWrapper.like("reagent_consumable_name", name);
        }

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFull = reagentConsumableInventoryService.getReagentConsumableInventoryFull(Wrappers.<ReagentConsumableInventory>query()
                .eq((StrUtil.isNotBlank(category)), "category", category)
                .like((StrUtil.isNotBlank(name)), "reagent_consumable_name", name));

        return R.ok(reagentConsumableInventoryFull);
    }



    /**-------标准物质一览表打印*/
    /**
     * 标准物质一览表打印
     *
     * @param
     * @return
     */
    @ApiOperation(value = "标准物质一览表打印", notes = "标准物质一览表打印")
    @SysLog("标准物质一览表打印")
    @PostMapping("/print")
	@PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_print')")
    public void bizGetPDFInventory( HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) {
        System.out.println("bizApplyWord.................");
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        String id = IdWorker.get32UUID().toUpperCase();

        List<ReferenceMaterialVO> voList = referenceMaterialService.getReferenceMaterialList();

        if (voList.isEmpty()) {
            throw new RuntimeException("未查询到相关信息");
        }


        String applyFileName = "标准物质一览表-" + id;

        String pdfFilePath = "document" + "/" + "referenceMaterial" + "/" + id + "/" + applyFileName + ".pdf";
        try {
            //直接调用pdf方法
            reagentConsumableInventoryService.referenceMaterialTablePDF(voList, id, theHttpServletRequest, httpServletResponse);
            ossFile.fileGet(pdfFilePath, httpServletResponse.getOutputStream());
            httpServletResponse.setContentType(applyFileName);
        } catch (Exception e) {
            System.out.println(String.format("minioFile objectExist() Exception. %s", e.getLocalizedMessage()));
            e.printStackTrace();
        }
    }

    /**
     * 生成 QRCODE 图像
     *
     * @param code QRCODE字符串
     *
     * @return R<CaseEventVO>
     */
    @ApiOperation(value = "生成 QRCODE 图像", notes = "生成 QRCODE 图像")
    @GetMapping("/qrcode")
    public void genQRCode(String code, HttpServletResponse httpServletResponse) throws IOException {
        try {
            // QR Code
            httpServletResponse.setContentType("image/png");
            BufferedImage qrImage = QRCodeUtils.genQRCode(code, 150, 150);
            ImageIO.write(qrImage, "png", httpServletResponse.getOutputStream());
        } catch (Exception e) {
            httpServletResponse.sendError(501, e.toString());
        }
    }

    /**
     * 生成 BARCODE 图像
     *
     * @param code QRCODE字符串
     *
     * @return R<CaseEventVO>
     */
    @ApiOperation(value = "生成条形码", notes = "生成条形码")
    @GetMapping("/code")
    @PreAuthorize("@pms.hasAnyPermission('reagent_reagent_consumable_inventory_get')" )

    public String getBarCodeImageBase64( String code, HttpServletResponse httpServletResponse) throws IOException {


        return reagentConsumableInventoryService.buildCodeLabelContent(code);


    }

    /**
     * 生成 BARCODE 图像
     *
     * @param code QRCODE字符串
     *
     * @return R<CaseEventVO>
     */
    @ApiOperation(value = "扫码录入物品编码", notes = "扫码录入物品编码")
    @PutMapping("/code")
    @PreAuthorize("@pms.hasPermission('reagent_reagent_consumable_inventory_get')")
    public R<String> setCode( String id,String code,HttpServletResponse httpServletResponse)  {

        reagentConsumableInventoryService.setCode(id,code);

        return R.ok("录入成功");
    }



}
