package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.ComplianceCheckDTO;
import digital.laboratory.platform.reagent.dto.OutgoingContentsDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.DeliveryRegistrationFormMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.DeliveryRegistrationFormVO;
import digital.laboratory.platform.reagent.vo.OutgoingContentsVO;
import digital.laboratory.platform.reagent.vo.ReagentConsumablesSetVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (试剂耗材出库登记表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材出库登记表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class DeliveryRegistrationFormServiceImpl extends ServiceImpl<DeliveryRegistrationFormMapper, DeliveryRegistrationForm> implements DeliveryRegistrationFormService {

    @Autowired
    private ApplicationForUseService applicationForUseService;

    @Autowired
    private ReagentConsumablesSetService reagentConsumablesSetService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private OutgoingContentsService outgoingContentsService;

    @Autowired
    private DeliveryRegistrationFormService deliveryRegistrationFormService;

    @Autowired
    private ComplianceCheckService complianceCheckService;

    @Autowired
    private StandardMaterialApplicationService standardMaterialApplicationService;

    @Autowired
    private RequisitionRecordService requisitionRecordService;

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Autowired
    private SolutionUseFormService solutionUseFormService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private BatchDetailsService batchDetailsService;

    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;

    @Override
    @Transactional//创建出库任务
    public DeliveryRegistrationForm addFrom(String applicationForUseId) {

        ApplicationForUse byId = applicationForUseService.getById(applicationForUseId);
        //创建出库登记表
        DeliveryRegistrationForm deliveryRegistrationForm = new DeliveryRegistrationForm();

        deliveryRegistrationForm.setId(IdWorker.get32UUID().toUpperCase());
        deliveryRegistrationForm.setOutgoingApplicantId(byId.getRecipientId());
        deliveryRegistrationForm.setStatus(0);
        byId.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());

        LambdaQueryWrapper<ReagentConsumablesSet> reagentConsumablesSetLambdaQueryWrapper = new LambdaQueryWrapper<>();
        //得到领用集合
        reagentConsumablesSetLambdaQueryWrapper.eq(ReagentConsumablesSet::getApplicationForUseId, byId.getId());

        List<ReagentConsumablesSet> list = reagentConsumablesSetService.list(reagentConsumablesSetLambdaQueryWrapper);

        List<OutgoingContents> outgoingContentsList = new ArrayList<>();
        //循环领用集合，创建出库集合
        for (ReagentConsumablesSet reagentConsumablesSet : list) {

            OutgoingContents outgoingContents = new OutgoingContents();

            ReagentConsumables one = reagentConsumablesService.getById(reagentConsumablesSet.getReagentConsumableId());

            if (one.getCategory().equals("标准物质") | one.getCategory().equals("标准储备溶液")) {

                LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();

                ReferenceMaterial referenceMaterial = referenceMaterialService.getById(reagentConsumablesSet.getReferenceMaterialId());

                outgoingContents.setReferenceMaterialId(referenceMaterial.getId());

                outgoingContents.setId(IdWorker.get32UUID().toUpperCase());
                outgoingContents.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());
                outgoingContents.setQuantity(reagentConsumablesSet.getQuantity());
                outgoingContents.setRemarks(reagentConsumablesSet.getRemarks());
                outgoingContents.setOutboundUse(reagentConsumablesSet.getPurpose());
                outgoingContents.setReagentConsumableId(reagentConsumablesSet.getReagentConsumableId());
                outgoingContents.setReagentConsumableType(one.getCategory());
                outgoingContents.setReferenceMaterialNumber(referenceMaterial.getNumber());
                outgoingContents.setBatchDetailsId(reagentConsumablesSet.getBatchDetailsId());
                outgoingContents.setLocation(referenceMaterial.getLocation());
                outgoingContents.setLatticeId(referenceMaterial.getLatticeId());
                outgoingContents.setBoxId(referenceMaterial.getBoxId());
                outgoingContents.setCode(referenceMaterial.getCode());

                outgoingContentsList.add(outgoingContents);
            }else {

                BatchDetails byId1 = batchDetailsService.getById(reagentConsumablesSet.getBatchDetailsId());

                outgoingContents.setId(IdWorker.get32UUID().toUpperCase());
                outgoingContents.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());
                outgoingContents.setQuantity(reagentConsumablesSet.getQuantity());
                outgoingContents.setRemarks(reagentConsumablesSet.getRemarks());
                outgoingContents.setOutboundUse(reagentConsumablesSet.getPurpose());
                outgoingContents.setReagentConsumableId(reagentConsumablesSet.getReagentConsumableId());
                outgoingContents.setReagentConsumableType(one.getCategory());
                outgoingContents.setBatchDetailsId(reagentConsumablesSet.getBatchDetailsId());
                outgoingContents.setLocation(byId1.getLocation());
                outgoingContents.setLatticeId(byId1.getLatticeId());
                outgoingContents.setBoxId(byId1.getBoxId());
                outgoingContents.setCode(one.getCode());
                outgoingContentsList.add(outgoingContents);

            }
            reagentConsumablesSet.setOutgoingContentId(outgoingContents.getId());

            reagentConsumablesSetService.updateById(reagentConsumablesSet);


        }
        if (deliveryRegistrationFormService.save(deliveryRegistrationForm) && outgoingContentsService.saveBatch(outgoingContentsList) &
                applicationForUseService.updateById(byId)) {

            return deliveryRegistrationForm;
        } else throw new RuntimeException(String.format("创建出库任务失败"));
    }

    @Override//提交出库登记表
    @Transactional
    public DeliveryRegistrationForm commitForm(List<OutgoingContentsDTO> outgoingContentsDTOS, DLPUser dlpUser) {
        //得到出库登记表
        DeliveryRegistrationForm byId = deliveryRegistrationFormService.getById(outgoingContentsDTOS.get(0).getDeliveryRegistrationFormId());

        byId.setStatus(1);

        byId.setDepositorId(dlpUser.getId());

        LambdaQueryWrapper<OutgoingContents> outgoingContentsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        outgoingContentsLambdaQueryWrapper.eq(OutgoingContents::getDeliveryRegistrationFormId, byId.getId());
        //删除修改之前的内容(防止领用为X种，但实际出库只有n种，导致领用出库不一致)
        outgoingContentsService.removeBatchByIds(outgoingContentsService.list(outgoingContentsLambdaQueryWrapper));

        List<StandardMaterialApplication> standardMaterialApplications = new ArrayList<>();

        List<OutgoingContents> outgoingContentsList = new ArrayList<>();

        LambdaQueryWrapper<ApplicationForUse> applicationForUseLambdaQueryWrapper = new LambdaQueryWrapper<>();

        applicationForUseLambdaQueryWrapper.eq(ApplicationForUse::getDeliveryRegistrationFormId, byId.getId());
        //查询领用申请表
        ApplicationForUse one = applicationForUseService.getOne(applicationForUseLambdaQueryWrapper);

        one.setClaimCode("");

        one.setStatus(2);

        applicationForUseService.updateById(one);

        for (OutgoingContentsDTO outgoingContentsDTO : outgoingContentsDTOS) {
            //传入出库数量，区分出库数量与领用数量
            ReagentConsumablesSet reagentConsumablesSet = reagentConsumablesSetService.getOne(Wrappers.<ReagentConsumablesSet>query().
                    eq("outgoing_content_id", outgoingContentsDTO.getId()));

            reagentConsumablesSet.setQuantityDelivered(outgoingContentsDTO.getQuantity());

            reagentConsumablesSetService.updateById(reagentConsumablesSet);

            OutgoingContents outgoingContents = new OutgoingContents();

            BeanUtils.copyProperties(outgoingContentsDTO, outgoingContents);

            outgoingContents.setId(IdWorker.get32UUID().toUpperCase());

            ReagentConsumables reagentConsumables = reagentConsumablesService.getById(outgoingContents.getReagentConsumableId());

            outgoingContents.setReagentConsumableType(reagentConsumables.getCategory());

            outgoingContents.setDeliveryRegistrationFormId(byId.getId());

            outgoingContents.setReferenceMaterialNumber(outgoingContentsDTO.getNumber());

            if (reagentConsumables.getCategory().equals("标准物质") ) {
                //生成标准物质领用/归还登记表
                StandardMaterialApplication standardMaterialApplication = new StandardMaterialApplication();

                standardMaterialApplication.setStandardMaterialApplicationId(IdWorker.get32UUID().toUpperCase());
                standardMaterialApplication.setPurpose(outgoingContents.getOutboundUse());
                standardMaterialApplication.setDateOfClaim(LocalDateTime.now());
                standardMaterialApplication.setRecipientId(one.getRecipientId());
                standardMaterialApplication.setReferenceSubstanceId(reagentConsumables.getReagentConsumableId());
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());
                standardMaterialApplication.setRequisitionedQuantity(outgoingContents.getQuantity());
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());
                standardMaterialApplication.setApplicationForUseId(one.getId());
                standardMaterialApplication.setReferenceMaterialId(outgoingContentsDTO.getReferenceMaterialId());
                standardMaterialApplication.setAdministratorId(dlpUser.getId());
                standardMaterialApplication.setReferenceMaterialNumber(outgoingContentsDTO.getNumber());
                standardMaterialApplications.add(standardMaterialApplication);
                standardMaterialApplication.setIsReturn(outgoingContentsDTO.getOutboundUse());
                standardMaterialApplication.setDeliveryRegistrationFormId(byId.getId());
                standardMaterialApplication.setReferenceSubstanceName(reagentConsumables.getReagentConsumableName());
                if (outgoingContentsDTO.getOutboundUse() == 2) {
                    standardMaterialApplication.setStatus(-1);
                    standardMaterialApplication.setPurposeAndQuantity(outgoingContents.getQuantity().toString());
                } else {
                    standardMaterialApplication.setStatus(0);
                }
                ReferenceMaterial byId1 = referenceMaterialService.getById(outgoingContentsDTO.getReferenceMaterialId());

                byId1.setStatus(-3);

                referenceMaterialService.updateById(byId1);

                BatchDetails batchDetails = batchDetailsService.getById(outgoingContentsDTO.getBatchDetailsId());

                if (batchDetails!=null){

                    batchDetails.setQuantity(batchDetails.getQuantity()-outgoingContentsDTO.getQuantity());

                    if (batchDetails.getQuantity()==0){

                        batchDetails.setServiceStatus(-1);
                    }

                    batchDetailsService.updateById(batchDetails);}

                //查找出对应的仓库信息，将库存量减少
                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.reduceById(outgoingContents.getReagentConsumableId(), outgoingContents.getQuantity());


                if (reagentConsumableInventory.getTotalQuantity() < 0) {

                    throw new RuntimeException(String.format("出库数量不能超过库存量"));
                }

            } else if (reagentConsumables.getCategory().equals("标准储备溶液") ) {

                StandardReserveSolution reference_id = standardReserveSolutionService.getOne(Wrappers.<StandardReserveSolution>query().eq("reference_id", outgoingContentsDTO.getReferenceMaterialId()));

                //创建标准物质领用/归还登记表
                StandardMaterialApplication standardMaterialApplication = new StandardMaterialApplication();
                standardMaterialApplication.setReferenceMaterialNumber(outgoingContentsDTO.getNumber());
                standardMaterialApplication.setStandardMaterialApplicationId(IdWorker.get32UUID().toUpperCase());
                standardMaterialApplication.setPurpose(outgoingContents.getOutboundUse());
                standardMaterialApplication.setDateOfClaim(LocalDateTime.now());
                standardMaterialApplication.setRecipientId(one.getRecipientId());
                standardMaterialApplication.setReferenceSubstanceId(reagentConsumables.getReagentConsumableId());
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());
                standardMaterialApplication.setRequisitionedQuantity(outgoingContents.getQuantity());
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());
                standardMaterialApplication.setApplicationForUseId(one.getId());
                standardMaterialApplication.setDeliveryRegistrationFormId(byId.getId());
                standardMaterialApplication.setReferenceSubstanceName(reference_id.getSolutionName());


                if (outgoingContentsDTO.getOutboundUse() == 2) {
                    standardMaterialApplication.setPurposeAndQuantity(outgoingContents.getQuantity().toString());
                    standardMaterialApplication.setStatus(1);
                } else {
                    standardMaterialApplication.setStatus(0);
                }
                standardMaterialApplication.setAdministratorId(dlpUser.getId());
                standardMaterialApplication.setReferenceMaterialId(outgoingContentsDTO.getReferenceMaterialId());
                standardMaterialApplications.add(standardMaterialApplication);
                //录入标准储备溶液使用记录
                SolutionUseForm solutionUseForm = solutionUseFormService.useById(outgoingContentsDTO.getNumber(), dlpUser);

                solutionUseForm.setStandardMaterialApplicationId(standardMaterialApplication.getStandardMaterialApplicationId());

                solutionUseFormService.updateById(solutionUseForm);
                //通过标准物质ID，查找出对应的仓库信息，将库存量减少
                String referenceMaterialId = outgoingContentsDTO.getReferenceMaterialId();

                ReferenceMaterial referenceMaterial = referenceMaterialService.getById(referenceMaterialId);

                referenceMaterial.setStatus(-3);

                referenceMaterialService.updateById(referenceMaterial);

                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.getById(referenceMaterial.getReagentConsumableInventoryId());

                reagentConsumableInventory.setTotalQuantity(reagentConsumableInventory.getTotalQuantity() - outgoingContentsDTO.getQuantity());

                BatchDetails batchDetails = batchDetailsService.getById(outgoingContentsDTO.getBatchDetailsId());

                if (batchDetails!=null){

                batchDetails.setQuantity(batchDetails.getQuantity()-outgoingContentsDTO.getQuantity());

                    if (batchDetails.getQuantity()==0){

                        batchDetails.setServiceStatus(-1);
                    }
                batchDetailsService.updateById(batchDetails);}
                if (reagentConsumableInventory.getTotalQuantity() < 0) {
                    throw new RuntimeException(String.format("出库数量不能超过库存量"));
                }

            } else {
                RequisitionRecord requisitionRecord = requisitionRecordService.addRecord(outgoingContents);
                //查找对应的批次ID，将批次数量减少
                BatchDetails batchDetails = batchDetailsService.getById(outgoingContentsDTO.getBatchDetailsId());
                /*
                * 若某个批次的物品已被领用完毕，则改变该批次的状态即可，就不会显示在领用页面*/
                if (batchDetails!=null){
                    //减少对应批次数量
                    batchDetails.setQuantity(batchDetails.getQuantity()-outgoingContentsDTO.getQuantity());

                    if (batchDetails.getQuantity()==0){

                        batchDetails.setServiceStatus(-1);
                    }

                    batchDetailsService.updateById(batchDetails);}

                requisitionRecord.setDateOfClaim(LocalDateTime.now());
                requisitionRecord.setRecipientId(one.getRecipientId());
                requisitionRecord.setId(byId.getId());


                requisitionRecordService.updateById(requisitionRecord);
                //查找出对应的仓库信息，将库存量减少
                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.reduceById(outgoingContents.getReagentConsumableId(), outgoingContents.getQuantity());

                if (reagentConsumableInventory.getTotalQuantity() < 0) {
                    throw new RuntimeException(String.format("出库数量不能超过库存量"));
                }
            }
            //如果用途是用作案件，则需创建符合性检查记录表
            if (outgoingContents.getOutboundUse() == 0) {
                ComplianceCheckDTO complianceCheckDTO = new ComplianceCheckDTO();
                complianceCheckDTO.setBrand(reagentConsumables.getBrand());
                complianceCheckDTO.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                complianceCheckDTO.setSpecificationAndModel(reagentConsumables.getSpecificationAndModel());
                complianceCheckDTO.setComplianceCheckId(IdWorker.get32UUID().toUpperCase());
                complianceCheckDTO.setReagentConsumableNumber(outgoingContentsDTO.getNumber());
                complianceCheckDTO.setBatchDetailsId(batchDetailsService.getById(outgoingContentsDTO.getBatchDetailsId()).getBatchDetailsId());
                complianceCheckService.addCheck(complianceCheckDTO, dlpUser);
            }
            outgoingContentsList.add(outgoingContents);
        }
        if (standardMaterialApplications.size() != 0) {
            standardMaterialApplicationService.saveBatch(standardMaterialApplications);
        }
        if (outgoingContentsService.saveBatch(outgoingContentsList)&& deliveryRegistrationFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("提交失败"));
    }

    @Override//分页查询
    public Page<DeliveryRegistrationFormVO> getDeliveryRegistrationFormVOPage(Page page, QueryWrapper<DeliveryRegistrationForm> qw) {

        Page deliveryRegistrationFormVOPage = baseMapper.getDeliveryRegistrationFormVOPage(page, qw);

        return deliveryRegistrationFormVOPage;
    }

    @Override//通过ID查询出库内容
    public DeliveryRegistrationFormVO getDeliveryRegistrationFormVOById(String deliveryRegistrationFormId) {

        DeliveryRegistrationFormVO deliveryRegistrationFormVOById = baseMapper.getDeliveryRegistrationFormVOById(deliveryRegistrationFormId);

        LambdaQueryWrapper<OutgoingContents> outgoingContentsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        outgoingContentsLambdaQueryWrapper.eq(OutgoingContents::getDeliveryRegistrationFormId, deliveryRegistrationFormVOById.getId());

        List<OutgoingContents> list = outgoingContentsService.list(outgoingContentsLambdaQueryWrapper);

        List<OutgoingContentsVO> outgoingContentsVOS = new ArrayList<>();

        if (list.size() != 0) {

            for (OutgoingContents outgoingContents : list) {

                OutgoingContentsVO outgoingContentsVO = new OutgoingContentsVO();

                BeanUtils.copyProperties(outgoingContents, outgoingContentsVO);

                LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

                reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, outgoingContents.getReagentConsumableId());

                ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

                outgoingContentsVO.setReagentConsumableInventory(one);

                outgoingContentsVOS.add(outgoingContentsVO);
            }

            deliveryRegistrationFormVOById.setOutgoingContentsList(outgoingContentsVOS);
        }

        return deliveryRegistrationFormVOById;
    }
    @Override//通过ID查询出库内容
    public DeliveryRegistrationFormVO getDeliveryRegistrationFormVOByCode(String claimCode) {

        if (!StrUtil.isNotBlank(claimCode)){

            return null;
        }

        ApplicationForUse applicationForUse = applicationForUseService.getOne(Wrappers.<ApplicationForUse>query()
                .eq("claim_code", claimCode));

        if (applicationForUse==null){

            return null;
        }

        DeliveryRegistrationFormVO deliveryRegistrationFormVOById = baseMapper.getDeliveryRegistrationFormVOById(applicationForUse.getDeliveryRegistrationFormId());

        LambdaQueryWrapper<OutgoingContents> outgoingContentsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        outgoingContentsLambdaQueryWrapper.eq(OutgoingContents::getDeliveryRegistrationFormId, deliveryRegistrationFormVOById.getId());

        List<OutgoingContents> list = outgoingContentsService.list(outgoingContentsLambdaQueryWrapper);

        List<OutgoingContentsVO> outgoingContentsVOS = new ArrayList<>();

        if (list.size() != 0) {

            for (OutgoingContents outgoingContents : list) {

                OutgoingContentsVO outgoingContentsVO = new OutgoingContentsVO();

                BeanUtils.copyProperties(outgoingContents, outgoingContentsVO);

                LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

                reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, outgoingContents.getReagentConsumableId());

                ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

                outgoingContentsVO.setReagentConsumableInventory(one);

                outgoingContentsVOS.add(outgoingContentsVO);
            }

            deliveryRegistrationFormVOById.setOutgoingContentsList(outgoingContentsVOS);
        }

        return deliveryRegistrationFormVOById;
    }

}
