package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.PurchaseListDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.PurchaseListMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.*;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * (采购清单)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购清单) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class PurchaseListServiceImpl extends ServiceImpl<PurchaseListMapper, PurchaseList> implements PurchaseListService {

    @Autowired
    private PurchaseListService purchaseListService;
    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Autowired
    private WarehousingRecordFormService warehousingRecordFormService;

    @Autowired
    private CatalogueDetailsService catalogueDetailsService;

    @Autowired
    private WarehousingContentService warehousingContentService;

    @Autowired
    private DecentralizeDetailsService decentralizeDetailsService;

    @Autowired
    private SupplierInformationService supplierInformationService;
    @Autowired
    private DecentralizedRequestService decentralizedRequestService;
    @Autowired
    private PurchasingPlanService purchasingPlanService;

    @Autowired
    private ProcurementContentService procurementContentService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private CentralizedRequestService centralizedRequestService;

    @Autowired
    private DetailsOfCentralizedService detailsOfCentralizedService;
    @Autowired
    private OssFile ossFile;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;
    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Override//通过ID查找清单
    public PurchaseListVO getPurchaseList(String purchaseListId) {

        PurchaseListVO purchaseListVO = new PurchaseListVO();

        PurchaseList byId = purchaseListService.getById(purchaseListId);

        List<PurchaseListDetailsVO> purchaseListDetailsVOList = purchaseListDetailsService.getPurchaseListDetailsVOList(purchaseListId);

        BeanUtils.copyProperties(byId, purchaseListVO);

        for (PurchaseListDetailsVO purchaseListDetailsVO : purchaseListDetailsVOList) {

            LambdaQueryWrapper<SupplierInformation> supplierInformationLambdaQueryWrapper = new LambdaQueryWrapper<>();

            if (purchaseListDetailsVO.getSupplierId() != null) {

                supplierInformationLambdaQueryWrapper.eq(SupplierInformation::getId, purchaseListDetailsVO.getSupplierId());

                SupplierInformation one = supplierInformationService.getOne(supplierInformationLambdaQueryWrapper);

                purchaseListDetailsVO.setSupplierName(one.getSupplierName());
            }
        }

        purchaseListVO.setPurchaseListDetailsVOS(purchaseListDetailsVOList);

        return purchaseListVO;
    }

    @Override
    @Transactional
    public void editById(List<PurchaseListDTO> purchaseListDTOList) {

        for (PurchaseListDTO purchaseListDTO : purchaseListDTOList) {

            PurchaseListDetails byId = purchaseListDetailsService.getById(purchaseListDTO.getId());

            byId.setSupplierId(purchaseListDTO.getSupplierId());

            purchaseListDetailsService.updateById(byId);
        }
    }

    @Transactional
    @Override//提交采购清单
    public PurchaseList commitById(String id) {

        PurchaseList purchaseList = purchaseListService.getById(id);
        //将集中采购与分散采购申请的状态改变
        if (purchaseList.getType().equals("采购计划")) {

            PurchasingPlan plan = purchasingPlanService.getOne(Wrappers.<PurchasingPlan>query()
                    .eq("purchase_list_id", id));

            PurchasingPlanVO purchasingPlanVO = purchasingPlanService.getPurchasingPlanVO(plan.getPurchasingPlanId());
             List<CentralizedRequest> centralizedRequestList = centralizedRequestService.list(Wrappers.<CentralizedRequest>query().eq("purchasing_plan_id", plan.getPurchasingPlanId()));
            for (CentralizedRequest centralizedRequest : centralizedRequestList) {
                centralizedRequest.setStatus(4);
                centralizedRequestService.updateById(centralizedRequest);
            }
        } else {
            DecentralizedRequest purchase_list_id = decentralizedRequestService.getOne(Wrappers.<DecentralizedRequest>query()
                    .eq("purchase_list_id", id));
            purchase_list_id.setStatus(6);
            decentralizedRequestService.updateById(purchase_list_id);
        }
        purchaseList.setStatus(1);

        LambdaQueryWrapper<PurchaseListDetails> purchaseListDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

        purchaseListDetailsLambdaQueryWrapper.eq(PurchaseListDetails::getPurchaseListId, purchaseList.getId());

        List<PurchaseListDetails> purchaseListDetailsList = purchaseListDetailsService.list(purchaseListDetailsLambdaQueryWrapper);
        //创建入库内容集合
        List<WarehousingContent> warehousingContentList = new ArrayList<>();
        //创建入库记录表
        WarehousingRecordForm warehousingRecordForm = new WarehousingRecordForm();
        //录入入库记录表信息
        warehousingRecordForm.setId(IdWorker.get32UUID().toUpperCase());

        warehousingRecordForm.setPurchaseListId(purchaseList.getId());

        warehousingRecordForm.setPurchaseNumber(LocalDate.now() + "入库任务");

        warehousingRecordForm.setStatus(0);

        for (PurchaseListDetails purchaseListDetails : purchaseListDetailsList) {

            ReagentConsumables byId = reagentConsumablesService.getById(purchaseListDetails.getReagentConsumableId());
            ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(Wrappers.<ReagentConsumableInventory>query()
                    .eq("reagent_consumable_id", byId.getReagentConsumableId()));


            WarehousingContent warehousingContent = new WarehousingContent();
            //完善入库内容信息
            warehousingContent.setId(IdWorker.get32UUID().toUpperCase());

            warehousingContent.setReagentConsumableName(byId.getReagentConsumableName());

            warehousingContent.setTotalQuantity(purchaseListDetails.getPurchaseQuantity());

            warehousingContent.setReagentConsumableId(purchaseListDetails.getReagentConsumableId());

            warehousingContent.setWarehousingRecordFormId(warehousingRecordForm.getId());

            warehousingContent.setSupplierId(purchaseListDetails.getSupplierId());

            warehousingContent.setWarehousingQuantity(0);

            warehousingContent.setCode(byId.getCode());

            warehousingContent.setPurchaseListDetailsId(purchaseListDetails.getId());

            if (one!=null){
            warehousingContent.setWarningValue(one.getWarningValue());
            }
            LambdaQueryWrapper<CatalogueDetails> catalogueDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

            catalogueDetailsLambdaQueryWrapper.eq(CatalogueDetails::getReagentConsumableId, warehousingContent.getReagentConsumableId());

            List<CatalogueDetails> list = catalogueDetailsService.list(catalogueDetailsLambdaQueryWrapper);

            warehousingContentList.add(warehousingContent);
        }

        if (purchaseListService.updateById(purchaseList) & warehousingRecordFormService.save(warehousingRecordForm)
                & warehousingContentService.saveBatch(warehousingContentList)) {
            return purchaseList;
        } else {
            return null;
        }
    }

    @Transactional
    @Override//录入采购清单明细
    public PurchaseListDetailsVO addDetails(PurchaseListDTO purchaseListDTO) {

        String purchaseListId = purchaseListDTO.getPurchaseListId();

        PurchaseList byId = purchaseListService.getById(purchaseListId);

        PurchaseListDetails purchaseListDetails = purchaseListDetailsService.getById(purchaseListDTO.getId());

        purchaseListDetails.setSupplierId(purchaseListDTO.getSupplierId());

        purchaseListDetailsService.updateById(purchaseListDetails);

        PurchaseListDetailsVO purchaseListDetailsVO = purchaseListDetailsService.getPurchaseListDetailsVO(purchaseListDetails.getId());

        return purchaseListDetailsVO;
    }

    @Override
    public IPage<PurchaseListPageVO> getPurchaseListVOPage(IPage<PurchaseList> page, @Param(Constants.WRAPPER) QueryWrapper<PurchaseList> qw) {

        IPage<PurchaseListPageVO> purchaseListVOPage = baseMapper.getPurchaseListVOPage(page, qw);

        return purchaseListVOPage;
    }


}
