package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteTemplate2htmlService;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.config.PageUtils;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.handle.AppStartupRunner;
import digital.laboratory.platform.reagent.mapper.ReagentConsumableInventoryMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.*;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 试剂耗材库存服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 试剂耗材库存 服务实现类
 */
@Service
@SuppressWarnings("all")
public class ReagentConsumableInventoryServiceImpl extends ServiceImpl<ReagentConsumableInventoryMapper, ReagentConsumableInventory> implements ReagentConsumableInventoryService {

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;
    @Autowired
    private BatchDetailsService batchDetailsService;
    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private RemoteTemplate2htmlService remoteTemplate2htmlService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private FixedValueReportService fixedValueReportService;

    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Autowired
    private OssFile ossFile;

    @Override//试剂耗材/标准物质标准物质管理列表
    public IPage<ReagentConsumableInventoryVO> getReagentConsumableInventoryRMVOList(IPage<ReagentConsumableInventory> page, QueryWrapper<ReagentConsumableInventory> qw) {

        IPage<ReagentConsumableInventoryVO> reagentConsumableInventoryVOPage = baseMapper.getReagentConsumableInventoryVOPage(page, qw);

        List<ReagentConsumableInventoryVO> records = reagentConsumableInventoryVOPage.getRecords();

        for (ReagentConsumableInventoryVO reagentConsumableInventoryVO : records) {

            List<BatchDetailsVO> batchDetailsVOList = batchDetailsService.getBatchDetailsList(reagentConsumableInventoryVO.getReagentConsumableInventoryId());
            //赋值试剂耗材批次明细
            reagentConsumableInventoryVO.setBatchDetailsVOS(batchDetailsVOList);
        }
        return reagentConsumableInventoryVOPage;
    }


    @Override//入库增加库存
    public ReagentConsumableInventory addById(String reagentConsumableId, Integer quantity) {

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumableId);

        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        one.setTotalQuantity(one.getTotalQuantity() + quantity);


        if (reagentConsumableInventoryService.updateById(one)) {

            return one;
        } else throw new RuntimeException(String.format("库存添加失败"));
    }

    @Override//领用减少库存
    public ReagentConsumableInventory reduceById(String reagentConsumableId, Integer quantity) {

        LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();

        reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, reagentConsumableId);

        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);

        one.setTotalQuantity(one.getTotalQuantity() - quantity);

        if (reagentConsumableInventoryService.updateById(one)) {

            return one;
        } else throw new RuntimeException(String.format("库存减少失败"));
    }


    @Override//分页查询标准物质
    public Page<ReagentConsumableInventoryFullVO> getAllList(Integer current, Integer size, QueryWrapper<ReagentConsumableInventory> qw, Integer status) {

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList = new ArrayList<>();

        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(qw);

        ArrayList<ReferenceMaterial> referenceMaterialList = new ArrayList<>();
        //通过仓库类信息，查找对应的该类所有标准物质
        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            if (reagentConsumableInventory.getTotalQuantity() > 0) {

                LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();

                if (status != null) {

                    referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getStatus, status).
                            eq(ReferenceMaterial::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId())
                            .ne(ReferenceMaterial::getStatus, -4);
                } else {
                    referenceMaterialLambdaQueryWrapper.eq(ReferenceMaterial::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId())
                            .ne(ReferenceMaterial::getStatus, -4);
                }
                List<ReferenceMaterial> list1 = referenceMaterialService.list(referenceMaterialLambdaQueryWrapper);

                referenceMaterialList.addAll(list1);
            }
        }

        if (referenceMaterialList.size() != 0) {
            //提供多余字段
            for (ReferenceMaterial referenceMaterial : referenceMaterialList) {

                ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                ReagentConsumableInventory byId = reagentConsumableInventoryService.getById(referenceMaterial.getReagentConsumableInventoryId());

                BeanUtils.copyProperties(byId, reagentConsumableInventoryFullVO);

                reagentConsumableInventoryFullVO.setNumber(referenceMaterial.getNumber());

                reagentConsumableInventoryFullVO.setBatchDetailsId(referenceMaterial.getBatchDetailsId());

                BatchDetails batchDetails = batchDetailsService.getById(referenceMaterial.getBatchDetailsId());

                SupplierInformation supplierInformation = supplierInformationService.getById(batchDetails.getSupplierId());

                LambdaQueryWrapper<StandardReserveSolution> standardReserveSolutionLambdaQueryWrapper = new LambdaQueryWrapper<>();

                standardReserveSolutionLambdaQueryWrapper.eq(StandardReserveSolution::getReferenceId, referenceMaterial.getId());

                StandardReserveSolution one1 = standardReserveSolutionService.getOne(standardReserveSolutionLambdaQueryWrapper);

                reagentConsumableInventoryFullVO.setSupplierName(supplierInformation.getSupplierName());
                reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());
                reagentConsumableInventoryFullVO.setBatchNumber(batchDetails.getBatchNumber());
                reagentConsumableInventoryFullVO.setManufacturerId(batchDetails.getSupplierId());
                reagentConsumableInventoryFullVO.setPurchaseTime(batchDetails.getPurchaseTime());
                reagentConsumableInventoryFullVO.setReferenceMaterialId(referenceMaterial.getId());
                reagentConsumableInventoryFullVO.setReferenceMaterialStatus(referenceMaterial.getStatus());
                reagentConsumableInventoryFullVO.setStorageLife(Integer.valueOf(batchDetails.getLimitDate()));
                if (one1 != null) {

                    reagentConsumableInventoryFullVO.setConfigurationConcentration(one1.getConfigurationConcentration());
                    reagentConsumableInventoryFullVO.setConfigurationDate(one1.getConfigurationDate());
                    reagentConsumableInventoryFullVO.setSolutionNumbering(referenceMaterial.getNumber());
                    reagentConsumableInventoryFullVO.setValidityPeriod(one1.getValidityPeriod());
                }
                reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);
            }
        }

        PageUtils pageUtils = new PageUtils();

        Page pages = pageUtils.getPages(current, size, reagentConsumableInventoryFullVOList);

        return pages;
    }


    //分页查询试剂耗材
    @Override
    public IPage<ReagentConsumableInventoryFullVO> getAllRM(IPage<ReagentConsumableInventory> page, QueryWrapper<ReagentConsumableInventory> qw) {

        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(qw);

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOS = new ArrayList<>();

        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            List<BatchDetails> list1 = batchDetailsService.list(Wrappers.<BatchDetails>query()
                    .eq("reagent_consumable_inventory_id", reagentConsumableInventory.getReagentConsumableInventoryId())
                    .eq("service_status", 1));

            //将不同批次的物品展示出来
            for (BatchDetails batchDetails : list1) {

                ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                reagentConsumableInventoryFullVO.setSupplierName(supplierInformationService.getById(batchDetails.getSupplierId()).getSupplierName());

                reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                reagentConsumableInventoryFullVO.setBatchDetailsId(batchDetails.getBatchDetailsId());

                reagentConsumableInventoryFullVO.setTotalQuantity(batchDetails.getQuantity());

                reagentConsumableInventoryFullVOS.add(reagentConsumableInventoryFullVO);
            }
        }
        PageUtils pageUtils = new PageUtils();

        Page pages = pageUtils.getPages((int) page.getCurrent(), (int) page.getSize(), reagentConsumableInventoryFullVOS);

        return pages;

    }


    @Override
    public List<ReagentConsumableInventoryFullVO> getReagentConsumableInventoryFull(QueryWrapper<ReagentConsumableInventory> qw) {
        List<ReagentConsumableInventory> list = reagentConsumableInventoryService.list(qw);
//        return list;

        List<ReagentConsumableInventoryFullVO> reagentConsumableInventoryFullVOList = new ArrayList<>();

        //找出对应的仓库信息
        for (ReagentConsumableInventory reagentConsumableInventory : list) {

            LambdaQueryWrapper<BatchDetails> batchDetailsLambdaQueryWrapper = new LambdaQueryWrapper<>();

            batchDetailsLambdaQueryWrapper.eq(BatchDetails::getReagentConsumableInventoryId, reagentConsumableInventory.getReagentConsumableInventoryId())
                    .or().eq(BatchDetails::getServiceStatus, 1);

            List<BatchDetails> list1 = batchDetailsService.list(batchDetailsLambdaQueryWrapper);
            //将不同批次的物品展示出来
            for (BatchDetails batchDetails : list1) {

                ReagentConsumableInventoryFullVO reagentConsumableInventoryFullVO = new ReagentConsumableInventoryFullVO();

                BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableInventoryFullVO);

                SupplierInformation byId = supplierInformationService.getById(batchDetails.getSupplierId());

                reagentConsumableInventoryFullVO.setSupplierName(byId.getSupplierName());

                reagentConsumableInventoryFullVO.setBatch(batchDetails.getBatch());

                reagentConsumableInventoryFullVO.setBatchDetailsId(batchDetails.getBatchDetailsId());

                reagentConsumableInventoryFullVO.setTotalQuantity(batchDetails.getQuantity());

                reagentConsumableInventoryFullVOList.add(reagentConsumableInventoryFullVO);
            }
        }
        return reagentConsumableInventoryFullVOList;
    }


    @Override
    public void referenceMaterialTablePDF(List<ReferenceMaterialVO> voList, String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("referenceMaterialTablePDF.................");

        int x = 1;

        for (ReferenceMaterialVO referenceMaterialVO : voList) {

            referenceMaterialVO.setI(x);
            x++;
        }
        //-----------------------------
        // 生成 word 版本的 采购目录


        String applyFileName = "标准物质一览表-" + id;

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ossFile.fileGet("template" + "/" + "标准物质一览表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);
        builder.bind("voList", policy).build();

        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("voList", voList);
                    put("id", id);
                    put("size", 1);
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "referenceMaterial" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }

    //试剂/耗材扫描物品编码进行持久化
    @Override
    public void setCode(String id, String code) {

        ReagentConsumables byId = reagentConsumablesService.getById(id);

        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(Wrappers.<ReagentConsumableInventory>query()
                .eq("reagent_consumable_id", id));

        if (byId != null & one != null) {
            byId.setCode(code);
            one.setCode(code);

            reagentConsumableInventoryService.updateById(one);
            reagentConsumablesService.updateById(byId);
        }

    }

    @Override
//标准物质条形码打印
    public String buildCodeLabelContent(String id) {

        ReferenceMaterial referenceMaterialServiceById = referenceMaterialService.getById(id);

        Map<String, Object> data = new HashMap<>();

        referenceMaterialServiceById.setCode(LocalDateTimeUtil.format(LocalDateTime.now(),"yyyyMMddHHmmssSSS"));

        referenceMaterialService.updateById(referenceMaterialServiceById);
        ReagentConsumables byId = reagentConsumablesService.getById(referenceMaterialServiceById.getReagentConsumableId());
        String reagentConsumableName = byId.getReagentConsumableName();
        ReferenceMaterialVO referenceMaterial = new ReferenceMaterialVO();
        BeanUtils.copyProperties(referenceMaterialServiceById,referenceMaterial);
        referenceMaterial.setCodeName(reagentConsumableName+":"+referenceMaterialServiceById.getCode());
        data.put("referenceMaterial", referenceMaterial);

        String templateFileName = AppStartupRunner.getCfg( "labelTemplate_standardSubstance");

        return remoteTemplate2htmlService.getHtml(templateFileName,data);

    }
}
