package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.entity.ApplicationForUse;
import digital.laboratory.platform.reagent.entity.OutgoingContents;
import digital.laboratory.platform.reagent.entity.ReagentConsumables;
import digital.laboratory.platform.reagent.entity.RequisitionRecord;
import digital.laboratory.platform.reagent.mapper.RequisitionRecordMapper;
import digital.laboratory.platform.reagent.service.ApplicationForUseService;
import digital.laboratory.platform.reagent.service.DeliveryRegistrationFormService;
import digital.laboratory.platform.reagent.service.ReagentConsumablesService;
import digital.laboratory.platform.reagent.service.RequisitionRecordService;
import digital.laboratory.platform.reagent.vo.RequisitionRecordVO;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;

/**
 * (试剂耗材领用记录表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材领用记录表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class RequisitionRecordServiceImpl extends ServiceImpl<RequisitionRecordMapper, RequisitionRecord> implements RequisitionRecordService {

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private RequisitionRecordService requisitionRecordService;
    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Autowired
    private ApplicationForUseService applicationForUseService;

    @Autowired
    private DeliveryRegistrationFormService deliveryRegistrationFormService;

    @Autowired
    private OssFile ossFile;
    @Override
    public RequisitionRecord addRecord(OutgoingContents outgoingContents){


        RequisitionRecord requisitionRecord = new RequisitionRecord();

        ReagentConsumables byId = reagentConsumablesService.getById(outgoingContents.getReagentConsumableId());

        requisitionRecord.setRequisitionRecordId(IdWorker.get32UUID().toUpperCase());
        requisitionRecord.setReagentConsumableId(byId.getReagentConsumableId());
        requisitionRecord.setRemarks(outgoingContents.getRemarks());
        requisitionRecord.setDrawingAmount(outgoingContents.getQuantity());
        requisitionRecord.setSpecificationAndModel(byId.getSpecificationAndModel());

        if (requisitionRecordService.save(requisitionRecord)){
            return requisitionRecord;
        }else throw new RuntimeException(String.format("试剂耗材领用记录表创建失败"));

    }


    @Override
    public IPage<RequisitionRecordVO> getRequisitionRecordVOPage(IPage<RequisitionRecord> page, QueryWrapper<RequisitionRecord> qw){

        IPage<RequisitionRecordVO> requisitionRecordVOPage = baseMapper.getRequisitionRecordVOPage(page, qw);

        return requisitionRecordVOPage;


    }

    @Override
    public List<RequisitionRecordVO> getRequisitionRecordVO(String id){

        ApplicationForUse byId = applicationForUseService.getById(id);

        String deliveryRegistrationFormId = byId.getDeliveryRegistrationFormId();

        List<RequisitionRecordVO> requisitionRecordVO = baseMapper.getRequisitionRecordVO(deliveryRegistrationFormId);

        return requisitionRecordVO;
    }

    @Override
    public void rquisitionRecordTablePDF(List<RequisitionRecordVO> voList,String id, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {

        System.out.println("RequisitionRecordTablePDF.................");


        //-----------------------------
        // 生成 word 版本的 采购目录

        String applyFileName = "试剂耗材领用记录表-"+id;

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ossFile.fileGet("template" + "/" + "试剂耗材领用记录表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        int x = 1;
        for (RequisitionRecordVO requisitionRecordVO : voList) {

            LocalDateTime dateOfClaim = requisitionRecordVO.getDateOfClaim();

            requisitionRecordVO.setDateOfClaims(LocalDateTimeUtil.format(dateOfClaim,"yyyy年MM月dd日"));

            requisitionRecordVO.setI(x);

            x++;
        }

        Configure config = Configure.builder().
                bind("voList", policy).build();
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, config).render(
                new HashMap<String, Object>() {{
                    put("voList", voList);
                    put("id",id);
                    put("size",1);

                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "requisitionRecord" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }
}
