package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.StandardMaterialApplicationDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.StandardMaterialApplicationMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.StandardMaterialApplicationVO;

import digital.laboratory.platform.sys.feign.RemoteCabinetService;
import feign.Response;
import lombok.RequiredArgsConstructor;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;


import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;

/**
 * (标准物质领用/归还登记表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质领用 / 归还登记表) 服务实现类
 */
@Service
@RequiredArgsConstructor
@SuppressWarnings("all")
public class StandardMaterialApplicationServiceImpl extends ServiceImpl<StandardMaterialApplicationMapper, StandardMaterialApplication> implements StandardMaterialApplicationService {

    @Autowired
    private StandardMaterialApplicationService standardMaterialApplicationService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;
    @Autowired
    private SolutionUseFormService solutionUseFormService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private BatchDetailsService batchDetailsService;

    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;

    @Autowired
    private OssFile ossFile;

    @Autowired

    private  RemoteCabinetService remoteCabinetService;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Override//归还标准物质
   @Transactional
    public StandardMaterialApplication returnById(StandardMaterialApplicationDTO standardMaterialApplicationDTO) {

        StandardMaterialApplication byId = standardMaterialApplicationService.getById(standardMaterialApplicationDTO.getStandardMaterialApplicationId());

        byId.setPurposeAndQuantity(standardMaterialApplicationDTO.getPurposeAndQuantity());
        byId.setDateOfReturn(LocalDateTime.now());
        byId.setStatus(1);

        ReferenceMaterial referenceMaterial = referenceMaterialService.getById(byId.getReferenceMaterialId());

        remoteCabinetService.updateCabinet(standardMaterialApplicationDTO.getLatticeId(), referenceMaterial.getId(), "1");

        ReagentConsumables reagentConsumables = reagentConsumablesService.getById(referenceMaterial.getReagentConsumableId());

        referenceMaterial.setLocation(standardMaterialApplicationDTO.getLocation());
        referenceMaterial.setLocation(standardMaterialApplicationDTO.getLocation());
        referenceMaterial.setBoxId(standardMaterialApplicationDTO.getBoxId());
        //若无需归还，则证明标准物质/标准储备溶液已经使用完毕，删除记录即可
        if (standardMaterialApplicationDTO.getOutOfUse()) {

            referenceMaterial.setStatus(-4);

            if (reagentConsumables.getCategory().equals("标准储备溶液")) {

                StandardReserveSolution solution_numbering = standardReserveSolutionService.getOne(Wrappers.<StandardReserveSolution>query().eq("reference_id", referenceMaterial.getId()));

                solution_numbering.setStatus(-1);

                standardReserveSolutionService.updateById(solution_numbering);
            }

        } else {
            referenceMaterial.setStatus(0);
            BatchDetails byId2 = batchDetailsService.getById(referenceMaterial.getBatchDetailsId());
            byId2.setQuantity(byId2.getQuantity() + 1);
            ReagentConsumableInventory byId1 = reagentConsumableInventoryService.getById(referenceMaterial.getReagentConsumableInventoryId());
            byId1.setTotalQuantity(byId1.getTotalQuantity() + 1);
            if (byId2.getQuantity()>0){
                byId2.setServiceStatus(1);
            }
            batchDetailsService.updateById(byId2);
            reagentConsumableInventoryService.updateById(byId1);

        }
        referenceMaterialService.updateById(referenceMaterial);

        LambdaQueryWrapper<SolutionUseForm> solutionUseFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

        solutionUseFormLambdaQueryWrapper.eq(SolutionUseForm::getStandardMaterialApplicationId, byId.getStandardMaterialApplicationId());

        SolutionUseForm one = solutionUseFormService.getOne(solutionUseFormLambdaQueryWrapper);
        //如果是标准储备溶液，则需补充完善标准储备溶液使用记录表的使用数量
        if (one != null) {
            one.setQuantityUsed(byId.getPurposeAndQuantity());
            solutionUseFormService.updateById(one);
        }
//
        if (standardMaterialApplicationService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("归还失败"));
    }

    @Override//领用记录
    public IPage<StandardMaterialApplicationVO> getPage(IPage<StandardMaterialApplication> page, QueryWrapper<StandardMaterialApplication> qw) {

        IPage<StandardMaterialApplicationVO> standardMaterialApplicationVOPage = baseMapper.getStandardMaterialApplicationVOPage(page, qw);

        return standardMaterialApplicationVOPage;

    }

    //归还记录
    @Override//归还记录
    public IPage<StandardMaterialApplicationVO> getList(IPage<StandardMaterialApplication> page, QueryWrapper<StandardMaterialApplication> qw) {

        IPage<StandardMaterialApplicationVO> standardMaterialApplicationVOIPage = baseMapper.getStandardMaterialApplicationVOList(page, qw);

        List<StandardMaterialApplicationVO> records = standardMaterialApplicationVOIPage.getRecords();

        for (StandardMaterialApplicationVO record : records) {

            ReagentConsumables byId = reagentConsumablesService.getById(record.getReferenceSubstanceId());

            record.setCategory(byId.getCategory());
            //将位置信息传入VO对象
            ReferenceMaterial byId1 = referenceMaterialService.getById(record.getReferenceMaterialId());

            record.setLocation(byId1.getLocation());
            record.setLatticeId(byId1.getLatticeId());
            record.setBoxId(byId1.getBoxId());

            LambdaQueryWrapper<StandardReserveSolution> standardReserveSolutionLambdaQueryWrapper = new LambdaQueryWrapper<>();

            standardReserveSolutionLambdaQueryWrapper.eq(StandardReserveSolution::getSolutionNumbering, record.getReferenceMaterialNumber());

            StandardReserveSolution one = standardReserveSolutionService.getOne(standardReserveSolutionLambdaQueryWrapper);

            record.setReagentConsumables(byId);

            if (one != null) {
                String configurationConcentration = one.getConfigurationConcentration();
                record.setConfigurationConcentration(configurationConcentration);

            }

        }
        return standardMaterialApplicationVOIPage;
    }


    @Override
    public void standardMaterialApplicationTablePDF(List<StandardMaterialApplicationVO> voList, String ids, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("standardMaterialApplicationTablePDF.................");

        int x = 1;
        for (StandardMaterialApplicationVO standardMaterialApplicationVO : voList) {

            standardMaterialApplicationVO.setDateOfClaims(LocalDateTimeUtil.format(standardMaterialApplicationVO.getDateOfClaim(), "yyyy年MM月dd日"));
            standardMaterialApplicationVO.setDateOfReturns(LocalDateTimeUtil.format(standardMaterialApplicationVO.getDateOfReturn(), "yyyy年MM月dd日"));
            standardMaterialApplicationVO.setX(x);
            if (standardMaterialApplicationVO.getPurpose() == 0) {
                standardMaterialApplicationVO.setPurposeAndQuantity("案件," + standardMaterialApplicationVO.getPurposeAndQuantity());

            } else {
                standardMaterialApplicationVO.setPurposeAndQuantity("科研," + standardMaterialApplicationVO.getPurposeAndQuantity());
            }
            x++;
        }

        //-----------------------------
        // 生成 word 版本的 采购目录


        String applyFileName = "标准物质领用归还登记表-" + ids;

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ossFile.fileGet("template" + "/" + "标准物质领用归还登记表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();


        Configure config = Configure.builder().
                bind("voList", policy).build();
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, config).render(
                new HashMap<String, Object>() {{
                    put("voList", voList);
                    put("ids", ids);
                    put("size", 1);
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "standardMaterialApplication" + "/" + ids + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }

    @Override
    public StandardMaterialApplicationVO getStandardMaterialApplicationVO(String id) {

        StandardMaterialApplicationVO standardMaterialApplicationVO = baseMapper.getStandardMaterialApplicationVO(id);

        ReferenceMaterial byId = referenceMaterialService.getById(standardMaterialApplicationVO.getReferenceMaterialId());

        standardMaterialApplicationVO.setLocation(byId.getLocation());
        standardMaterialApplicationVO.setBoxId(byId.getBoxId());
        standardMaterialApplicationVO.setLatticeId(byId.getLatticeId());

        return standardMaterialApplicationVO;
    }

}


