package digital.laboratory.platform.reagent.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.reagent.entity.CabinetForm;
import digital.laboratory.platform.reagent.entity.LatticeForm;
import digital.laboratory.platform.reagent.entity.StorageRoomForm;
import digital.laboratory.platform.reagent.mapper.StorageRoomFormMapper;
import digital.laboratory.platform.reagent.service.CabinetFormService;
import digital.laboratory.platform.reagent.service.LatticeFormService;
import digital.laboratory.platform.reagent.service.StorageRoomFormService;
import digital.laboratory.platform.reagent.vo.StorageRoomFormVO;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-22
 * @describe 服务实现类
 */
@Service
@RequiredArgsConstructor
public class StorageRoomFormServiceImpl extends ServiceImpl<StorageRoomFormMapper, StorageRoomForm> implements StorageRoomFormService {

    private StorageRoomFormService storageRoomFormService;

    private CabinetFormService cabinetFormService;

    private LatticeFormService latticeFormService;

    @Override//新增存储室信息
    public StorageRoomForm addById(StorageRoomForm storageRoomForm) {

        StorageRoomForm roomForm = new StorageRoomForm();

        BeanUtils.copyProperties(storageRoomForm, roomForm);

        roomForm.setId(IdWorker.get32UUID().toUpperCase());

        if (storageRoomFormService.save(roomForm)) {
            return roomForm;
        } else throw new RuntimeException(String.format("保存失败"));
    }

    @Override//修改存储室信息
    public StorageRoomForm editById(StorageRoomForm storageRoomForm) {

        StorageRoomForm byId = storageRoomFormService.getById(storageRoomForm.getId());

        BeanUtils.copyProperties(storageRoomForm, byId);

        if (storageRoomFormService.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("保存失败"));
    }
    @Override//通过ID查询存储室
    public StorageRoomFormVO getFormById(String  id){

        StorageRoomForm byId = storageRoomFormService.getById(id);

        LambdaQueryWrapper<CabinetForm> cabinetFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

        cabinetFormLambdaQueryWrapper.eq(CabinetForm::getStorageRoomFormId,id);

        List<CabinetForm> list = cabinetFormService.list(cabinetFormLambdaQueryWrapper);

        StorageRoomFormVO storageRoomFormVO = new StorageRoomFormVO();

        BeanUtils.copyProperties(byId,storageRoomFormVO);

        storageRoomFormVO.setCabinetFormList(list);

        return storageRoomFormVO;
    }
    @Override
    @Transactional
    public Boolean delById(String id){

        StorageRoomForm byId = storageRoomFormService.getById(id);

        LambdaQueryWrapper<CabinetForm> cabinetFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

        cabinetFormLambdaQueryWrapper.eq(CabinetForm::getStorageRoomFormId,id);

        List<CabinetForm> list = cabinetFormService.list(cabinetFormLambdaQueryWrapper);
        //删除格子信息
        for (CabinetForm cabinetForm : list) {

            LambdaQueryWrapper<LatticeForm> latticeFormLambdaQueryWrapper = new LambdaQueryWrapper<>();

            latticeFormLambdaQueryWrapper.eq(LatticeForm::getCabinetFormId,cabinetForm.getId());

            List<LatticeForm> latticeFormList = latticeFormService.list(latticeFormLambdaQueryWrapper);

            for (LatticeForm latticeForm : latticeFormList) {

                if (latticeForm.getStatus()==1){
                    throw new RuntimeException(String.format("当前柜子有物品存放，不允许删除"));
                }
            }

            latticeFormService.removeBatchByIds(latticeFormList);
        }

        if (cabinetFormService.removeBatchByIds(list)&&storageRoomFormService.removeById(byId)){
            return true;
        }else throw new RuntimeException(String.format("删除失败"));
    }


}
