package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.PageUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.config.PageUtils;
import digital.laboratory.platform.reagent.dto.WarehousingRecordFormDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.WarehousingRecordFormMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.*;
import digital.laboratory.platform.sys.feign.RemoteCabinetService;
import feign.Response;
import io.seata.spring.annotation.GlobalTransactional;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

/**
 * 签收记录表服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 签收记录表 服务实现类
 */
@Service
@SuppressWarnings("all")
public class WarehousingRecordFormServiceImpl extends ServiceImpl<WarehousingRecordFormMapper, WarehousingRecordForm> implements WarehousingRecordFormService {
    @Autowired
    private WarehousingRecordFormService warehousingRecordFormService;

    @Autowired
    private WarehousingContentService warehousingContentService;

    @Autowired
    private WarehousingBatchListService warehousingBatchListService;

    @Autowired
    private StorageRegistrationFormService storageRegistrationFormService;

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private RemoteCabinetService remoteCabinetService;


    @Autowired
    private BatchDetailsService batchDetailsService;

    @Autowired
    private AcceptanceRecordFormService acceptanceRecordFormService;
    @Autowired
    private ProvideServicesOrSuppliesService provideServicesOrSuppliesService;

    @Autowired
    private EvaluationFormService evaluationFormService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Autowired
    private ProcurementContentService procurementContentService;
    @Autowired

    private DetailsOfCentralizedService detailsOfCentralizedService;

    @Autowired
    private DecentralizeDetailsService decentralizeDetailsService;

    @Autowired
    private PurchasingPlanService purchasingPlanService;

    @Autowired
    private CentralizedRequestService centralizedRequestService;

    @Autowired
    private DecentralizedRequestService decentralizedRequestService;

    @Autowired
    private PurchaseListService purchaseListService;

    @Override//查看采购入库
    public WarehousingRecordFormVO getWarehousingRecordFormVO(String warehousingFormId) {

        WarehousingRecordForm byId = warehousingRecordFormService.getById(warehousingFormId);

        WarehousingRecordFormVO warehousingRecordFormVO = new WarehousingRecordFormVO();

        BeanUtils.copyProperties(byId, warehousingRecordFormVO);

        List<WarehousingContentVO> warehousingContentVOList = warehousingContentService.getWarehousingContentVOList(warehousingFormId);

        warehousingRecordFormVO.setWarehousingContentVOList(warehousingContentVOList);

        return warehousingRecordFormVO;
    }


    @GlobalTransactional
    @Override//录入入库明细
    public WarehousingRecordFormVO addFormById(List<WarehousingRecordFormDTO> warehousingRecordFormDTOList, DLPUser dlpUser) {

        for (WarehousingRecordFormDTO warehousingRecordFormDTO : warehousingRecordFormDTOList) {

            if (warehousingRecordFormDTO.getLatticeId().isEmpty() & warehousingRecordFormDTO.getLocation().isEmpty() &
                    warehousingRecordFormDTO.getBoxId().isEmpty()) {

                throw new RuntimeException(String.format("请选择存放位置后再进行提交"));
            }
        }

        boolean flag = true;

        int i = 0;
        //签收内容循环
        for (WarehousingRecordFormDTO warehousingRecordFormDTO : warehousingRecordFormDTOList) {

            String latticeId = warehousingRecordFormDTO.getLatticeId();
            //录入签收批次信息
            WarehousingBatchList warehousingBatchList = new WarehousingBatchList();

            BeanUtils.copyProperties(warehousingRecordFormDTO, warehousingBatchList);

            warehousingBatchList.setId(IdWorker.get32UUID().toUpperCase());

            warehousingBatchList.setDepositorId(dlpUser.getId());

            warehousingBatchList.setDateOfReceipt(LocalDateTime.now());

            warehousingBatchList.setLocation(warehousingRecordFormDTO.getLocation());

            LambdaQueryWrapper<WarehousingBatchList> warehousingBatchListLambdaQueryWrapper = new LambdaQueryWrapper<>();

            warehousingBatchListLambdaQueryWrapper.eq(WarehousingBatchList::getWarehousingContentId, warehousingBatchList.getWarehousingContentId());
            //通过判断签收内容表里的签收批次表数量，进行批次字段的赋值
            warehousingBatchList.setBatch(warehousingBatchListService.list(warehousingBatchListLambdaQueryWrapper).size() + 1);

            WarehousingContent byId = warehousingContentService.getById(warehousingBatchList.getWarehousingContentId());

            byId.setWarehousingQuantity(byId.getWarehousingQuantity() + warehousingBatchList.getQuantity());
            //判断采购数量与签收数量
            if (byId.getWarehousingQuantity() > byId.getTotalQuantity()) {
                throw new RuntimeException(String.format("入库数量不能大于采购数量"));
            }

            warehousingContentService.updateById(byId);


            ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(Wrappers.<ReagentConsumableInventory>query().eq("reagent_consumable_id", byId.getReagentConsumableId()));
            //录入仓库批次信息，若仓库不存在该物品，则新增该物品的库存信息
            if (one == null) {

                ReagentConsumableInventory reagentConsumableInventory = new ReagentConsumableInventory();

                LambdaQueryWrapper<ReagentConsumables> reagentConsumablesLambdaQueryWrapper = new LambdaQueryWrapper<>();

                reagentConsumablesLambdaQueryWrapper.eq(ReagentConsumables::getReagentConsumableId, byId.getReagentConsumableId());

                ReagentConsumables reagentConsumables = reagentConsumablesService.getOne(reagentConsumablesLambdaQueryWrapper);

                BeanUtils.copyProperties(reagentConsumables, reagentConsumableInventory);

                reagentConsumableInventory.setReagentConsumableInventoryId(IdWorker.get32UUID().toUpperCase());
                reagentConsumableInventory.setStatus(1);
                reagentConsumableInventory.setCreateTime(LocalDateTime.now());
                reagentConsumableInventory.setUpdateTime(LocalDateTime.now());
                reagentConsumableInventory.setWarningValue(warehousingRecordFormDTO.getWarningValue());
                //如果供应商未提供物品编码，则需在入库时扫描标签录入编码
                if (warehousingRecordFormDTO.getCode() != null) {
                    reagentConsumableInventory.setCode(warehousingRecordFormDTO.getCode());
                    reagentConsumables.setCode(warehousingRecordFormDTO.getCode());
                    reagentConsumablesService.updateById(reagentConsumables);
                }

                BatchDetails batchDetails = new BatchDetails();

                BeanUtils.copyProperties(warehousingRecordFormDTO, batchDetails);
                batchDetails.setBatchDetailsId(IdWorker.get32UUID().toUpperCase());
                batchDetails.setServiceStatus(1);
                batchDetails.setPurchaseTime(LocalDateTime.now());
                batchDetails.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());
                batchDetails.setSupplierId(byId.getSupplierId());
                batchDetails.setBatch(1);
                batchDetails.setExpirationDate(warehousingRecordFormDTO.getExpirationDate());
                batchDetails.setQuantity(warehousingRecordFormDTO.getQuantity() * Integer.valueOf(reagentConsumables.getPackagedCopies()));

                warehousingBatchList.setBatchId(batchDetails.getBatchDetailsId());

                if (batchDetails.getQuantity() == null) {
                    batchDetails.setQuantity(0);
                }


                if (reagentConsumables.getCategory().equals("试剂") | reagentConsumables.getCategory().equals("耗材")) {
                    //更新格子信息（试剂耗材传入类ID）
                    remoteCabinetService.updateCabinet(latticeId, byId.getReagentConsumableId(), "1");
                    batchDetails.setLatticeId(warehousingRecordFormDTO.getLatticeId());
                    batchDetails.setLocation(warehousingRecordFormDTO.getLocation());
                    batchDetails.setBoxId(warehousingRecordFormDTO.getBoxId());
                }

                if (reagentConsumableInventory.getTotalQuantity() == null) {

                    reagentConsumableInventory.setTotalQuantity(0);
                }

                reagentConsumableInventory.setTotalQuantity(reagentConsumableInventory.getTotalQuantity() + batchDetails.getQuantity());

                //如果是标准物质，则赋值编号
                if (reagentConsumables.getCategory().equals("标准物质")) {


                    for (int j = 0; j < batchDetails.getQuantity(); j++) {

                        ReferenceMaterial referenceMaterial = new ReferenceMaterial();
                        referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
                        referenceMaterial.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                        referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());
                        referenceMaterial.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());
                        referenceMaterial.setStatus(0);
                        referenceMaterial.setLocation(warehousingRecordFormDTO.getLocation());
                        referenceMaterial.setLatticeId(warehousingRecordFormDTO.getLatticeId());
                        referenceMaterial.setCustodianName(warehousingRecordFormDTO.getCustodianName());
                        referenceMaterial.setBoxId(warehousingRecordFormDTO.getBoxId());


                        //更新格子信息（标准物质传入对象ID）
                        remoteCabinetService.updateCabinet(latticeId, referenceMaterial.getId(), "1");

                        Calendar calendar = Calendar.getInstance();

                        int year = calendar.get(Calendar.YEAR);

                        String location = warehousingRecordFormDTO.getLocation();

                        String prefix = reagentConsumables.getEnglishName() + "-" + year + "-" + location.charAt(1) + "-";

                        List<ReferenceMaterial> list = referenceMaterialService.list(Wrappers.<ReferenceMaterial>query()
                                .likeRight("number", prefix)
                                .orderByDesc("number"));

                        int newNo = 1;

                        if ((list != null) && (list.size() > 0)) {
                            ReferenceMaterial referenceMaterial1 = list.get(0);
                            String strMaxNo = StrUtil.removePrefixIgnoreCase(referenceMaterial1.getNumber(), prefix);
                            try {
                                int maxno = Integer.parseUnsignedInt(strMaxNo);
                                newNo = maxno + 1;
                            } catch (NumberFormatException e) {
                                // 如果后缀有非数字, 则无视之, 重头编码
                                newNo = 1;
                            }
                        }
                        referenceMaterial.setNumber(prefix + String.format("%03d", newNo));

                        referenceMaterialService.save(referenceMaterial);

                    }

                }
                reagentConsumableInventoryService.save(reagentConsumableInventory);

                batchDetailsService.save(batchDetails);
            } else {
                //如果仓库存在该物品

                if (warehousingRecordFormDTO.getCode() != null) {
                    one.setCode(warehousingRecordFormDTO.getCode());
                    ReagentConsumables byId1 = reagentConsumablesService.getById(one.getReagentConsumableId());
                    byId1.setCode(one.getCode());
                    reagentConsumablesService.updateById(byId1);
                }
                LambdaQueryWrapper<ReagentConsumables> reagentConsumablesLambdaQueryWrapper = new LambdaQueryWrapper<>();

                reagentConsumablesLambdaQueryWrapper.eq(ReagentConsumables::getReagentConsumableId, byId.getReagentConsumableId());

                ReagentConsumables reagentConsumables = reagentConsumablesService.getOne(reagentConsumablesLambdaQueryWrapper);

                BatchDetails batchDetails = new BatchDetails();

                BeanUtils.copyProperties(warehousingRecordFormDTO, batchDetails);

                batchDetails.setBatchDetailsId(IdWorker.get32UUID().toUpperCase());
                batchDetails.setServiceStatus(1);
                batchDetails.setPurchaseTime(LocalDateTime.now());
                batchDetails.setSupplierId(byId.getSupplierId());
                batchDetails.setExpirationDate(warehousingRecordFormDTO.getExpirationDate());
                batchDetails.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());
                batchDetails.setQuantity(warehousingRecordFormDTO.getQuantity() * Integer.valueOf(one.getPackagedCopies()));
                warehousingBatchList.setBatchId(batchDetails.getBatchDetailsId());

                List<BatchDetails> batchDetailsList = batchDetailsService.list(Wrappers.<BatchDetails>query().eq("reagent_consumable_inventory_id", one.getReagentConsumableInventoryId())
                        .eq("supplier_id", byId.getSupplierId()));

                if (batchDetailsList.size() == 0) {
                    batchDetails.setBatch(1);
                } else {
                    batchDetails.setBatch(batchDetailsList.size() + 1);
                }
                if (one.getCategory().equals("试剂") | one.getCategory().equals("耗材")) {
                    batchDetails.setLocation(warehousingRecordFormDTO.getLocation());
                    batchDetails.setLatticeId(warehousingRecordFormDTO.getLatticeId());
                    batchDetails.setBoxId(warehousingRecordFormDTO.getBoxId());
                    //更新格子信息（试剂耗材传入类ID）
                    remoteCabinetService.updateCabinet(latticeId, byId.getReagentConsumableId(), "1");
                }

                one.setTotalQuantity(one.getTotalQuantity() + batchDetails.getQuantity());
                one.setWarningValue(warehousingRecordFormDTO.getWarningValue());

                if (one.getCategory().equals("标准物质")) {

                    for (int j = 0; j < batchDetails.getQuantity(); j++) {

                        ReferenceMaterial referenceMaterial = new ReferenceMaterial();
                        referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
                        referenceMaterial.setReagentConsumableId(one.getReagentConsumableId());
                        referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());
                        referenceMaterial.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());
                        referenceMaterial.setStatus(0);
                        referenceMaterial.setLocation(warehousingRecordFormDTO.getLocation());
                        referenceMaterial.setLatticeId(warehousingRecordFormDTO.getLatticeId());
                        referenceMaterial.setBoxId(warehousingRecordFormDTO.getBoxId());
                        referenceMaterial.setCustodianName(warehousingRecordFormDTO.getCustodianName());
                        //更新格子信息（试剂耗材传入类ID）

                        remoteCabinetService.updateCabinet(latticeId, referenceMaterial.getId(), "1");
                        Calendar calendar = Calendar.getInstance();

                        int year = calendar.get(Calendar.YEAR);

                        String location = warehousingRecordFormDTO.getLocation();

                        String prefix = reagentConsumables.getEnglishName() + "-" + year + "-" + location.charAt(1) + "-";

                        List<ReferenceMaterial> list = referenceMaterialService.list(Wrappers.<ReferenceMaterial>query()
                                .likeRight("number", prefix)
                                .orderByDesc("number"));

                        int newNo = 1;

                        if ((list != null) && (list.size() > 0)) {
                            ReferenceMaterial referenceMaterial1 = list.get(0);
                            String strMaxNo = StrUtil.removePrefixIgnoreCase(referenceMaterial1.getNumber(), prefix);
                            try {
                                int maxno = Integer.parseUnsignedInt(strMaxNo);
                                newNo = maxno + 1;
                            } catch (NumberFormatException e) {
                                // 如果后缀有非数字, 则无视之, 重头编码
                                newNo = 1;
                            }
                        }
                        referenceMaterial.setNumber(prefix + String.format("%03d", newNo));

                        referenceMaterialService.save(referenceMaterial);
                    }

                }
                reagentConsumableInventoryService.updateById(one);

                batchDetailsService.save(batchDetails);
            }
            warehousingBatchListService.save(warehousingBatchList);
        }

        LambdaQueryWrapper<WarehousingContent> warehousingContentLambdaQueryWrapper = new LambdaQueryWrapper<>();

        WarehousingContent byId = warehousingContentService.getById(warehousingRecordFormDTOList.get(0).getWarehousingContentId());

        String warehousingRecordFormId = byId.getWarehousingRecordFormId();

        warehousingContentLambdaQueryWrapper.eq(WarehousingContent::getWarehousingRecordFormId, warehousingRecordFormId);

        List<WarehousingContent> list = warehousingContentService.list(warehousingContentLambdaQueryWrapper);

        WarehousingRecordForm warehousingRecordForm = warehousingRecordFormService.getById(warehousingRecordFormId);



//遍历采购内容，判断试剂耗材是否入库完毕
        for (WarehousingContent warehousingContent : list) {

            if (warehousingContent.getWarehousingQuantity() == warehousingContent.getTotalQuantity()) {

                warehousingContent.setStatus(1);

                warehousingContentService.updateById(warehousingContent);
                if (warehousingContent.getAcceptanceRecordFormId() == null) {

                    AcceptanceRecordForm acceptanceRecordForm = acceptanceRecordFormService.addForm(warehousingContent.getReagentConsumableId(), warehousingContent.getSupplierId());

                    warehousingContent.setAcceptanceRecordFormId(acceptanceRecordForm.getId());

                    warehousingContentService.updateById(warehousingContent);
                }

                i = i + 1;
            }
        }
        //判断该清单所有试剂耗材是否入库完成
        if (i == list.size()) {

            List<SupplierInformation> supplierInformations = new ArrayList<>();


            //创建供应商评价表
            /*
            通过循环签收内容，获得所有的供应商列表
            为每一个供应商创建一个评价表
            */
            for (WarehousingContent warehousingContent : list) {

                SupplierInformation supplierInformation = supplierInformationService.getById(warehousingContent.getSupplierId());

                if (!supplierInformations.contains(supplierInformation)) {
                    supplierInformations.add(supplierInformation);
                    EvaluationForm evaluationForm = evaluationFormService.addForm(supplierInformation.getId());
                    warehousingContent.setEvaluationFormId(evaluationForm.getId());
                    provideServicesOrSuppliesService.addById(evaluationForm.getId(), warehousingContent.getReagentConsumableId());
                } else {
                    LambdaQueryWrapper<EvaluationForm> evaluationFormLambdaQueryWrapper = new LambdaQueryWrapper<>();
                    evaluationFormLambdaQueryWrapper.eq(EvaluationForm::getSupplierInformationId, supplierInformation.getId())
                            .orderByDesc(EvaluationForm::getCreateTime);

                    List<EvaluationForm> list1 = evaluationFormService.list(evaluationFormLambdaQueryWrapper);

                    provideServicesOrSuppliesService.addById(list1.get(0).getId(), warehousingContent.getReagentConsumableId());
                }
            }

            warehousingRecordForm.setStatus(2);


        } else {
            warehousingRecordForm.setStatus(1);
        }

        warehousingRecordFormService.updateById(warehousingRecordForm);

        WarehousingRecordFormVO warehousingRecordFormVO = warehousingRecordFormService.getWarehousingRecordFormVO(warehousingRecordFormId);

        return warehousingRecordFormVO;
    }

    @Override
    public IPage<WarehousingRecordFormVO> getWarehousingRecordFormVOPage(Page<WarehousingRecordForm> page, QueryWrapper<WarehousingRecordForm> qw) {

        IPage<WarehousingRecordFormVO> warehousingRecordFormVOPage = baseMapper.getWarehousingRecordFormVOPage(page, qw);

        List<WarehousingRecordFormVO> records = warehousingRecordFormVOPage.getRecords();

        for (WarehousingRecordFormVO record : records) {

            List<WarehousingContent> warehousingContents = warehousingContentService.list(Wrappers.<WarehousingContent>query().eq("warehousing_record_form_id", record.getId()));

            for (WarehousingContent warehousingContent : warehousingContents) {

                Integer warehousingQuantity = warehousingContent.getWarehousingQuantity();
                Integer totalQuantity = warehousingContent.getTotalQuantity();
                if (record.getWarehousingQuantity() == null & record.getQuantityPurchased() == null) {
                    record.setWarehousingQuantity(0);
                    record.setQuantityPurchased(0);
                }
                record.setWarehousingQuantity(record.getWarehousingQuantity() + warehousingQuantity);
                record.setQuantityPurchased(record.getQuantityPurchased() + totalQuantity);
            }

        }
        return warehousingRecordFormVOPage;
    }

    public List<PurchaseRequestPrintVO> getPurchaseRequestPrint(String id) {

        WarehousingRecordForm byId = warehousingRecordFormService.getById(id);

        List<WarehousingContent> warehousingContentList = warehousingContentService.list(Wrappers.<WarehousingContent>query().eq("warehousing_record_form_id", id));

        PurchaseListDetails purchaseListDetails1 = purchaseListDetailsService.getById(warehousingContentList.get(0).getPurchaseListDetailsId());

        DecentralizedRequest request = decentralizedRequestService.getOne(Wrappers.<DecentralizedRequest>query().eq("purchase_list_id", purchaseListDetails1.getPurchaseListId()));

        if (request != null) {

            DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(request.getId());

            List<DecentralizeDetails> decentralizeDetailsVOList = requestById.getDecentralizeDetailsVOList();


        }


        for (WarehousingContent warehousingContent : warehousingContentList) {

            PurchaseListDetails purchaseListDetails = purchaseListDetailsService.getById(warehousingContent.getPurchaseListDetailsId());

            String purchaseListId = purchaseListDetails.getPurchaseListId();

            List<WarehousingBatchList> batchListList = warehousingBatchListService.list(Wrappers.<WarehousingBatchList>query().eq("warehousing_content_id", warehousingContent.getId()));

//            DecentralizedRequest request = decentralizedRequestService.getOne(Wrappers.<DecentralizedRequest>query().eq("purchase_list_id", purchaseListId));

            if (request != null) {

                DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(request.getId());

                List<DecentralizeDetails> decentralizeDetailsVOList = requestById.getDecentralizeDetailsVOList();

                for (DecentralizeDetails decentralizeDetail : decentralizeDetailsVOList) {

                    PurchaseRequestPrintVO purchaseRequestPrintVO = new PurchaseRequestPrintVO();

                    if (decentralizeDetail.getCategory().equals("标准物质")) {

                        purchaseRequestPrintVO.setRemarks(decentralizeDetail.getRemarks());
                        purchaseRequestPrintVO.setQuantity(decentralizeDetail.getQuantity());
                        purchaseRequestPrintVO.setPurpose(decentralizeDetail.getPurpose());
                        purchaseRequestPrintVO.setSubtotal(decentralizeDetail.getUnitPrice() * decentralizeDetail.getQuantity());
                        purchaseRequestPrintVO.setApplicantName(requestById.getApplicantName());
                        purchaseRequestPrintVO.setReagentConsumableName(decentralizeDetail.getReagentConsumableName());
//                    purchaseRequestPrintVO.setSupplierName();
                    }
                }


            }

            ProcurementContent procurementContent = procurementContentService.getById(purchaseListDetails.getProcurementContentId());


        }

        return
                null;

    }


}



